@@ -1,10 +1,10 @@
 package operator
 
 import (
+	"context"
 	"fmt"
 	"time"
 
-	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/apimachinery/pkg/util/wait"
@@ -33,24 +33,20 @@ const (
 	operatorVersionEnvName = "OPERATOR_IMAGE_VERSION"
 )
 
-var targetDeploymentNames = sets.NewString(
-	api.SignerControllerDeploymentName,
-	api.APIServiceInjectorDeploymentName,
-	api.ConfigMapInjectorDeploymentName,
-)
+var targetDeploymentNames = sets.NewString(api.ServiceCADeploymentName)
 
-func RunOperator(ctx *controllercmd.ControllerContext) error {
+func RunOperator(ctx context.Context, controllerContext *controllercmd.ControllerContext) error {
 
 	// This kube client uses protobuf, do not use it for CRs
-	kubeClient, err := kubernetes.NewForConfig(ctx.ProtoKubeConfig)
+	kubeClient, err := kubernetes.NewForConfig(controllerContext.ProtoKubeConfig)
 	if err != nil {
 		return err
 	}
-	operatorConfigClient, err := operatorv1client.NewForConfig(ctx.KubeConfig)
+	operatorConfigClient, err := operatorv1client.NewForConfig(controllerContext.KubeConfig)
 	if err != nil {
 		return err
 	}
-	configClient, err := configv1client.NewForConfig(ctx.KubeConfig)
+	configClient, err := configv1client.NewForConfig(controllerContext.KubeConfig)
 	if err != nil {
 		return err
 	}
@@ -86,7 +82,7 @@ func RunOperator(ctx *controllercmd.ControllerContext) error {
 		configInformers.Config().V1().ClusterOperators(),
 		operatorClient,
 		versionGetter,
-		ctx.EventRecorder,
+		controllerContext.EventRecorder,
 	)
 
 	resourceSyncController := resourcesynccontroller.NewResourceSyncController(
@@ -94,7 +90,7 @@ func RunOperator(ctx *controllercmd.ControllerContext) error {
 		kubeInformersForNamespaces,
 		v1helpers.CachedSecretGetter(kubeClient.CoreV1(), kubeInformersForNamespaces),
 		v1helpers.CachedConfigMapGetter(kubeClient.CoreV1(), kubeInformersForNamespaces),
-		ctx.EventRecorder,
+		controllerContext.EventRecorder,
 	)
 	if err := resourceSyncController.SyncConfigMap(
 		resourcesynccontroller.ResourceLocation{Namespace: operatorclient.GlobalMachineSpecifiedConfigNamespace, Name: clusterOperatorName},
@@ -110,68 +106,45 @@ func RunOperator(ctx *controllercmd.ControllerContext) error {
 		kubeClient.CoreV1(),
 		kubeClient.RbacV1(),
 		versionGetter,
-		ctx.EventRecorder,
+		controllerContext.EventRecorder,
 	)
 
-	operatorConfigInformers.Start(ctx.Done())
-	configInformers.Start(ctx.Done())
-	kubeInformersNamespaced.Start(ctx.Done())
-	kubeInformersForNamespaces.Start(ctx.Done())
+	stopChan := ctx.Done()
 
-	cleanupUnifiedDeployment(kubeClient, ctx.Done())
+	operatorConfigInformers.Start(stopChan)
+	configInformers.Start(stopChan)
+	kubeInformersNamespaced.Start(stopChan)
+	kubeInformersForNamespaces.Start(stopChan)
 
-	go operator.Run(ctx.Done())
-	go clusterOperatorStatus.Run(1, ctx.Done())
-	go resourceSyncController.Run(1, ctx.Done())
-
-	<-ctx.Done()
-	return fmt.Errorf("stopped")
-}
-
-// cleanupUnifiedDeployment removes resources associated with the unified service ca
-// controller deployment created by the 4.4 operator. This is intended to remove the
-// possibility of contention between the 4.4 deployment and the multiple deployments
-// managed by the 4.3 operator in the event of a downgrade from 4.4 to 4.3.
-func cleanupUnifiedDeployment(kubeClient *kubernetes.Clientset, stopCh <-chan struct{}) error {
-	controllerName := "service-ca"
-	namespace := operatorclient.TargetNamespace
-	configName := fmt.Sprintf("%s-config", controllerName)
-	lockName := fmt.Sprintf("%s-lock", controllerName)
-	saName := fmt.Sprintf("%s-sa", controllerName)
-	roleAndBindingName := fmt.Sprintf("system:openshift:controller:%s", controllerName)
-	delOpts := &metav1.DeleteOptions{}
-	deletionFuncs := []func() error{
-		// Delete deployment openshift-service-ca/<controller-name>
-		func() error { return kubeClient.AppsV1().Deployments(namespace).Delete(controllerName, delOpts) },
-		// Delete ClusterRole system:openshift:controller:{controller name}
-		func() error { return kubeClient.RbacV1().ClusterRoles().Delete(roleAndBindingName, delOpts) },
-		// Delete ClusterRole system:openshift:controller:{controller name}
-		func() error { return kubeClient.RbacV1().ClusterRoles().Delete(roleAndBindingName, delOpts) },
-		// Delete ClusterRoleBinding system:openshift:controller:{controller name}
-		func() error { return kubeClient.RbacV1().ClusterRoleBindings().Delete(roleAndBindingName, delOpts) },
-		// Delete ConfigMap openshift-service-ca/{controller name}-config
-		func() error { return kubeClient.CoreV1().ConfigMaps(namespace).Delete(configName, delOpts) },
-		// Delete ConfigMap openshift-service-ca/{controller name}-lock
-		func() error { return kubeClient.CoreV1().ConfigMaps(namespace).Delete(lockName, delOpts) },
-		// Delete Role openshift-service-ca/system:openshift:controller:{controller name}
-		func() error { return kubeClient.RbacV1().Roles(namespace).Delete(roleAndBindingName, delOpts) },
-		// Delete RoleBinding openshift-service-ca/system:openshift:controller:{controller name}
-		func() error { return kubeClient.RbacV1().RoleBindings(namespace).Delete(roleAndBindingName, delOpts) },
-		// Delete ServiceAccount openshift-service-ca/{controller name}-sa
-		func() error { return kubeClient.CoreV1().ServiceAccounts(namespace).Delete(saName, delOpts) },
-	}
+	// Poll every minute to ensure removal of 4.3 deployments that were replaced in 4.4
+	// with a unified deployment.
+	//
+	// This code can be removed in 4.5 since downgrade to 4.3 will no longer be possible.
+	deployClient := kubeClient.AppsV1().Deployments(operatorclient.TargetNamespace)
 	go wait.Until(func() {
-		klog.V(4).Infof("attempting removal of the unified service ca controller created by the 4.4 operator in namespace %q", operatorclient.TargetNamespace)
-		for _, deletionFunc := range deletionFuncs {
-			err := deletionFunc()
-			if apierrors.IsNotFound(err) {
-				// Already removed
-				continue
-			}
-			if err != nil {
-				klog.Errorf("Error deleting 4.4 resource: %v", err)
+		// List rather than blindly deleting so that there are max 1 request/minute
+		// instead of 3 (one for each 4.3 deployment).
+		deploys, err := deployClient.List(metav1.ListOptions{})
+		if err != nil {
+			klog.Warningf("Failed to list deployments when searching for 4.3 deployments to remove: %v", err)
+		}
+		if deploys == nil {
+			return
+		}
+		for _, deploy := range deploys.Items {
+			if api.IndependentDeploymentNames.Has(deploy.Name) {
+				err := deployClient.Delete(deploy.Name, &metav1.DeleteOptions{})
+				if err != nil {
+					klog.Warningf("Failed to delete 4.3 deployment: %v", err)
+				}
 			}
 		}
-	}, 20*time.Minute, stopCh)
-	return nil
+	}, 1*time.Minute, ctx.Done())
+
+	go operator.Run(stopChan)
+	go clusterOperatorStatus.Run(ctx, 1)
+	go resourceSyncController.Run(ctx, 1)
+
+	<-stopChan
+	return fmt.Errorf("stopped")
 }
